package de.unikoblenz.fgbks.base.domain;

import de.unikoblenz.fgbks.base.value.AbstractStringValueObject;

/**
 * A value object representing a name.
 */
public class Name extends AbstractStringValueObject {

  public static final Name NO_NAME = new Name("[no name]");
  /** Create a new copy of the given name object. */
  public Name(Name initialValue) {
    this(initialValue.getValue());
  }

  /** Create a name object with the given {@code initialValue}. */
  public Name(String initialValue) {
    super(initialValue);
  }

  @Override
  protected String validateAndNormalize(String newValue) {
    if (newValue.length() > getMaxLength()) {
      newValue = newValue.substring(0, getMaxLength() - 2) + "..";
    }
    return super.validateAndNormalize(newValue);
  }

  @Override
  protected Integer getMaxLength() {
    return 60;
  }
}
