package de.unikoblenz.fgbks.core.dmn.domain.vdmn;

import de.unikoblenz.fgbks.core.dmn.domain.ids.InputExpressionId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.InputId;
import java.util.List;
import org.camunda.bpm.model.dmn.instance.Input;

/**
 * This interface represents a input column in a {@link VDmnDecisionTable}. A column contains a list
 * of {@link VDmnInputValue}s.
 */
public interface VDmnInputColumn extends VDmnColumn {

  /**
   * Get the original Input.
   *
   * @return the {@link Input}
   */
  Input getInput();

  /**
   * Get the {@link InputExpressionId}.
   *
   * @return the {@link InputExpressionId}
   */
  default InputExpressionId getInputExpressionId() {
    return (InputExpressionId) getId();
  }

  /**
   * Get the {@link InputId}.
   *
   * @return the {@link InputId}
   */
  default InputId getInputId() {
    return (InputId) getColumnId();
  }

  /**
   * Get a list of {@link VDmnInputValue}s.
   *
   * @return the list of {@link VDmnInputValue}s
   */
  default List<VDmnInputValue> getVDmnInputValues() {
    return (List<VDmnInputValue>) (List<?>) getValues();
  }
}
