package de.unikoblenz.fgbks.core.dmn.verification.result.actions;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.PostConstruct;
import javax.enterprise.context.ApplicationScoped;

@ApplicationScoped
public class FixActionService {

  private boolean globalActive;
  private Map<ActionScope, Map<ActionType, Boolean>> allowedActions;

  public FixActionService() {
  }

  @PostConstruct
  private void init() {
    allowedActions = new HashMap<>();
    for (ActionScope actionScope : ActionScope.values()) {
      Map<ActionType, Boolean> types = new HashMap<>();
      allowedActions.put(actionScope, types);
      for (ActionType actionType : ActionType.values()) {
        types.put(actionType, true);
      }
    }
  }

  public boolean isGlobalActive() {
    return globalActive;
  }

  public void setGlobalActive(boolean globalActive) {
    this.globalActive = globalActive;
  }

  public Map<ActionScope, Map<ActionType, Boolean>> getAllowedActions() {
    return new HashMap<>(allowedActions);
  }

  public void setValue(ActionScope actionScope, ActionType actionType, Boolean value) {
    allowedActions.get(actionScope).put(actionType, value);
  }

  public List<ActionScope> getActionScopes() {
    return Arrays.asList(ActionScope.values());
  }

  public List<ActionType> getActionTypes() {
    return Arrays.asList(ActionType.values());
  }

  public boolean isValid(VerificationFix verificationFix) {
    if (!globalActive) {
      return false;
    }
    for (Action action : verificationFix.getActions()) {
      if (!allowedActions.get(action.getActionScope()).get(action.getActionType())) {
        return false;
      }
    }
    return true;
  }
}
