package de.unikoblenz.fgbks.core.dmn.verification.verifier.impl;

import static de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheckType.IS_EQUAL;
import static de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions.getRulesRowsStrings;
import static de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions.templateDecision;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.ActionScope.RULE;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.ActionType.DELETE;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.SHOW_RULES;

import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecision;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnRule;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntry.VerificationClassification;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntryElement;
import de.unikoblenz.fgbks.core.dmn.verification.result.actions.Action;
import de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.AbstractVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.DmnVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.IdenticalRuleVerification;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

@DmnVerifier(verifierType = IdenticalRuleVerification.class)
public class IdenticalRuleVerifier extends AbstractVerifier {

  @Override
  protected void doVerification() {
    for (VDmnDecision decision : dmnObjectContainer.getVDmnDefinition().getVDmnDecisions()) {
      checkDecisionTable(decision.getVDmnDecisionTable());
    }
  }

  private void checkDecisionTable(VDmnDecisionTable dmnDecisionTable) {
    if (dmnDecisionTable.getVDmnInputColumns().size() > 0) {
      checkForIdenticalRules(
          dmnDecisionTable,
          dmnDecisionTable.getVDmnInputColumns(),
          0,
          dmnDecisionTable.getVDmnRules());
    }
  }

  protected void checkForIdenticalRules(
      VDmnDecisionTable dmnDecisionTable,
      List<VDmnInputColumn> inputs,
      int i,
      List<VDmnRule> currentRules) {
    if (i == inputs.size()) {
      boolean sameConclusions = VDmnFunctions.sameConclusions(currentRules);
      currentRules.forEach(
          rule -> vreFactory.addElement(VerificationResultEntryElement.create(rule)));
      // Delete action, if output is the same
      if (sameConclusions) {
        VerificationFix.Builder verificationFixBuilder =
            VerificationFix.getBuilder().withFixName("Delete Duplicates");
        boolean first = true;
        for (VDmnRule rule : currentRules) {
          if (!first) {
            verificationFixBuilder.addAction(
                Action.getBuilder()
                    .withActionScope(RULE)
                    .withActionType(DELETE)
                    .addIdValue(rule.getVDmnDecision().getDecisionId())
                    .addIdValue(rule.getRuleId())
                    .build());
          }
          first = false;
        }
        vreFactory.addVerificationFix(verificationFixBuilder.build());
      }
      // Show action
      vreFactory
          .addVerificationFix(SHOW_RULES)
          .addToEntry(
              VerificationClassification.WARNING,
              templateDecision(dmnDecisionTable.getVDmnDecision())
                  + "Rules %s have identical inputs. The output is "
                  + (sameConclusions ? "the same." : "the same."),
              getRulesRowsStrings(currentRules));
    } else {
      List<VDmnInputValue> curInVals = new ArrayList<>();
      List<VDmnInputValue> sortInVals =
          VDmnFunctions.getColumnValuesInRules(inputs.get(i), currentRules);
      sortInVals.sort(Comparator.comparing(VDmnInputValue::getBoundary));
      VDmnInputValue lastVal = null;
      for (VDmnInputValue curVal : sortInVals) {
        if (lastVal != null) {
          if (curVal.getBoundary().checkWith(IS_EQUAL, lastVal.getBoundary())) {
            curInVals.add(lastVal);
          } else {
            if (curInVals.size() > 0) {
              curInVals.add(lastVal);
              checkForIdenticalRules(
                  dmnDecisionTable,
                  inputs,
                  i + 1,
                  curInVals.stream().map(VDmnValue::getVDmnRule).collect(Collectors.toList()));
            }
            curInVals.clear();
          }
        }
        lastVal = curVal;
      }
      if (curInVals.size() > 0) {
        curInVals.add(lastVal);
        checkForIdenticalRules(
            dmnDecisionTable,
            inputs,
            i + 1,
            curInVals.stream().map(VDmnValue::getVDmnRule).collect(Collectors.toList()));
      }
    }
  }
}
