package de.unikoblenz.fgbks.core.dmn.verification.verifier.impl;

import static de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions.getDecisionStringName;
import static de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntry.VerificationClassification.WARNING;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.ActionScope.INPUT_DATA;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.ActionType.DELETE;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.ActionType.UPDATE;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.SHOW_INPUT_DATA;

import de.unikoblenz.fgbks.base.domain.Name;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecision;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputData;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntryElement;
import de.unikoblenz.fgbks.core.dmn.verification.result.actions.Action;
import de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.AbstractVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.DmnVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.LonelyDataInputVerification;

@DmnVerifier(verifierType = LonelyDataInputVerification.class)
public class LonelyDataInputVerifier extends AbstractVerifier {

  private Name defaultName = new Name("[no name]");

  @Override
  protected void doVerification() {
    for (VDmnInputData i : dmnObjectContainer.getVDmnDefinition().getVDmnInputData()) {
      if (i.getInformationProvidingDecisions().isEmpty()) {
        // add element to result
        vreFactory
            .addElement(VerificationResultEntryElement.create(i))
            .addVerificationFix(SHOW_INPUT_DATA);
        // add delete action
        vreFactory.addVerificationFix(
            VerificationFix.getBuilder()
                .withFixName(new Name("Remove"))
                .addAction(
                    Action.getBuilder()
                        .withActionScope(INPUT_DATA)
                        .withActionType(DELETE)
                        .addIdValue(i.getInputDataId())
                        .build())
                .build());
        // search possible connections
        searchPossibleDecisions(i);
        // add to result
        vreFactory.addToEntry(
            WARNING,
            "Input data \"%s\" has no outgoing connections to at least one decision node.",
            i.getName().orElse(defaultName));
      }
    }
  }

  private void searchPossibleDecisions(VDmnInputData inputData) {
    if (inputData.getName().isPresent()) {
      for (VDmnDecision decision : dmnObjectContainer.getVDmnDefinition().getVDmnDecisions()) {
        for (VDmnInputColumn inputColumn : decision.getVDmnDecisionTable().getVDmnInputColumns()) {
          if (inputColumn.getName().isPresent()
              && inputColumn.getName().get().equals(inputData.getName().get())) {
            // Add verification fix to add this input Data to the decision
            vreFactory.addVerificationFix(
                VerificationFix.getBuilder()
                    .withFixName(new Name("Connect to " + getDecisionStringName(decision)))
                    .addAction(
                        Action.getBuilder()
                            .withActionScope(INPUT_DATA)
                            .withActionType(UPDATE)
                            .addIdValue(inputColumn.getVDmnDecision().getDecisionId())
                            .addIdValue(inputData.getInputDataId())
                            .addValue("name", inputColumn.getName().get().getValue())
                            .build())
                    .build());
          }
        }
      }
    }
  }
}
