package de.unikoblenz.fgbks.core.dmn.verification.verifier.impl;

import static de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheckType.IS_IN_CONTACT;
import static de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions.getColumnStringName;
import static de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions.templateDecision;
import static de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntry.VerificationClassification.WARNING;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.Action.ACTION_SHOW_INPUT_COLUMNS;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.Action.ACTION_SHOW_OUTPUT_ENTRIES;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.DEFAULT_SHOW_NAME;

import de.unikoblenz.fgbks.base.utils.boundary.Boundary;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecision;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntryElement;
import de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.AbstractVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.DmnVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.MissingInputValueVerification;

@DmnVerifier(verifierType = MissingInputValueVerification.class)
public class MissingInputValueVerifier extends AbstractVerifier {

  @Override
  protected void doVerification() {
    for (VDmnDecision decision : dmnObjectContainer.getVDmnDefinition().getVDmnDecisions()) {
      for (VDmnDecision nextDecision : decision.getInformationProvidingDecisions()) {
        checkMissingInputValues(decision, nextDecision);
      }
    }
  }

  private void checkMissingInputValues(VDmnDecision decision, VDmnDecision nextDecision) {
    // find the matching output -> input column
    for (VDmnOutputColumn outputColumn : decision.getVDmnDecisionTable().getVDmnOutputColumns()) {
      for (VDmnInputColumn inputColumn : nextDecision.getVDmnDecisionTable()
          .getVDmnInputColumns()) {
        if (outputColumn.getName().isPresent()
            && inputColumn.getName().isPresent()
            && outputColumn.getName().get().equals(inputColumn.getName().get())
            && outputColumn.getTypeRef().equals(inputColumn.getTypeRef())) {
          checkMissingInputValues(outputColumn, inputColumn);
        }
      }
    }
  }

  private void checkMissingInputValues(VDmnOutputColumn outputColumn, VDmnInputColumn inputColumn) {
    // check for each value in outputColumn, if it is used in the inputColumn
    outer:
    for (VDmnOutputValue outputValue : outputColumn.getVDmnOutputValues()) {
      if (!outputValue.getText().getValue().isEmpty()) {
        // create pseudo boundary for output value
        String outputText = outputValue.getText().getValue();
        if (outputColumn.getTypeRef() == VTypeRef.DOUBLE
            || outputColumn.getTypeRef() == VTypeRef.INTEGER
            || outputColumn.getTypeRef() == VTypeRef.LONG) {
          outputText = "= " + outputText;
        }
        Boundary b = outputColumn.getTypeRef().getBoundaryFromText(outputText);
        for (VDmnInputValue inputValue : inputColumn.getVDmnInputValues()) {
          if (b.checkWith(IS_IN_CONTACT, inputValue.getBoundary())) {
            continue outer;
          }
        }
        // no matching input value found for current output Value
        vreFactory
            .addElement(VerificationResultEntryElement.create(inputColumn))
            .addElement(VerificationResultEntryElement.create(outputValue))
            .addVerificationFix(
                VerificationFix.getBuilder()
                    .withFixName(DEFAULT_SHOW_NAME)
                    .addAction(ACTION_SHOW_OUTPUT_ENTRIES)
                    .addAction(ACTION_SHOW_INPUT_COLUMNS)
                    .build())
            .addToEntry(
                WARNING,
                templateDecision(inputColumn.getVDmnDecision())
                    + "Input column \"%s\" never handles the value '%s'.",
                getColumnStringName(inputColumn),
                outputValue.getText().getValue());
      }
    }
  }
}
